///////////////////////////////////////////////////////////////////////////////
// Name:        swWxGuiTesting/CppGuiTest/swWxGuiTestEvtSimHlpEvtHandler.cpp
// Author:      Reinhold Freder
// Created:     2004
// Copyright:   (c) 2005 Reinhold Freder
// Licence:     wxWindows licence
///////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "swWxGuiTestEvtSimHlpEvtHandler.h"
#endif

#include "swWxGuiTestEvtSimHlpEvtHandler.h"

#include "cppunit/TestAssert.h"

#include "wx/xrc/xmlres.h"

#include "wx/spinctrl.h"
#include "wx/notebook.h"

#include "swWxGuiTestHelper.h"
#include "swFrameFactory.h"
#include "swMdiFrameFactory.h"
#include "swToolBarRegistry.h"

BEGIN_EVENT_TABLE(swTst::WxGuiTestEvtSimHlpEvtHandler, wxEvtHandler)
    EVT_MENU( XRCID("MenuItem"), WxGuiTestEvtSimHlpEvtHandler::OnMenuItem )
    EVT_MENU( XRCID("CheckableMenuItem"), WxGuiTestEvtSimHlpEvtHandler::OnCheckableMenuItem )
    EVT_BUTTON( XRCID("Button"), WxGuiTestEvtSimHlpEvtHandler::OnButtonClick )
    EVT_TEXT( XRCID("TextCtrl"), WxGuiTestEvtSimHlpEvtHandler::OnTextCtrlChange )
    EVT_SPINCTRL( XRCID("SpinCtrlDbl"), WxGuiTestEvtSimHlpEvtHandler::OnSpinCtrlDblChange )
    EVT_TREE_SEL_CHANGED( XRCID("TreeCtrl"), WxGuiTestEvtSimHlpEvtHandler::OnTreeSelChange )
    EVT_TREE_ITEM_RIGHT_CLICK( XRCID("TreeCtrl"), WxGuiTestEvtSimHlpEvtHandler::OnTreeItemRightClick )
    EVT_NOTEBOOK_PAGE_CHANGED( XRCID("Notebook"), WxGuiTestEvtSimHlpEvtHandler::OnNotebookPageChange )
    EVT_CHOICE( XRCID("Choice"), WxGuiTestEvtSimHlpEvtHandler::OnChoiceSelChange )
    EVT_CHECKBOX( XRCID("Checkbox"), WxGuiTestEvtSimHlpEvtHandler::OnCheckboxChecking )
    EVT_RADIOBOX( XRCID("RadioBox"), WxGuiTestEvtSimHlpEvtHandler::OnRadioBoxSelChange )
    EVT_COMMAND_SCROLL( XRCID("Slider"), WxGuiTestEvtSimHlpEvtHandler::OnSliderChange )
    EVT_SPINCTRL( XRCID("SpinCtrl"), WxGuiTestEvtSimHlpEvtHandler::OnSpinCtrlChange )
    EVT_TOOL( XRCID("Tool"), WxGuiTestEvtSimHlpEvtHandler::OnToggleTool )
    EVT_TOOL( XRCID("ToggleTool"), WxGuiTestEvtSimHlpEvtHandler::OnToggleCheckableTool )
END_EVENT_TABLE()

namespace swTst {


WxGuiTestEvtSimHlpEvtHandler::WxGuiTestEvtSimHlpEvtHandler (wxFrame *frame)
{
    m_frame = frame;

    // Clear all flags:
    m_menuItemSelProcessed = false;
    m_chkMenuItemSelProcessed = false;
    m_buttonClickProcessed = false;
    m_textCtrlValChangeEvent = false;
    m_spinCtrlDblValChangeEvent = false;
    m_treeSelProcessed = false;
    m_treeItemRightClickProcessed = false;
    m_notebookPageSelProcessed = false;
    m_choiceSelProcessed = false;
    m_checkboxChkgProcessed = false;
    m_radioBoxSelProcessed = false;
    m_sliderChangeProcessed = false;
    m_spinCtrlChangeProcessed = false;
    m_toolTogglingProcessed = false;
    m_checkableToolTogglingProcessed = false;
}


WxGuiTestEvtSimHlpEvtHandler::~WxGuiTestEvtSimHlpEvtHandler ()
{
    // Nothing to do
}


bool WxGuiTestEvtSimHlpEvtHandler::HasMenuItemSelProcessed () const
{
    return m_menuItemSelProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::IsCheckableMenuItemChecked () const
{
    wxMenuBar *menuBar = m_frame->GetMenuBar ();
    CPPUNIT_ASSERT_MESSAGE ("Menubar not found", menuBar != NULL);
    int checkableMenuItemId = menuBar->FindMenuItem (_("Menu"), _("CheckableMenuItem"));
    CPPUNIT_ASSERT_MESSAGE ("CheckableMenuItem menu item ID not found",
            checkableMenuItemId != wxNOT_FOUND);
    wxMenu* menu = menuBar->GetMenu (0);
    CPPUNIT_ASSERT_MESSAGE ("Menu menu not found", menu != NULL);
    wxMenuItem *item = menuBar->FindItem (checkableMenuItemId, &menu);
    CPPUNIT_ASSERT_MESSAGE ("MenuItem menu item not found", item != NULL);

    CPPUNIT_ASSERT_MESSAGE ("Check state invalid",
            item->IsChecked () == m_chkMenuItemSelProcessed);

    return item->IsChecked ();
}


bool WxGuiTestEvtSimHlpEvtHandler::HasButtonClickProcessed () const
{
    return m_buttonClickProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasTextCtrlValChgProcessed () const
{
    return m_textCtrlValChangeEvent;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasSpinCtrlDblValChgWithoutEvtProcessed () const
{
    return m_spinCtrlDblValChangeEvent;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasTreeItemSelProcessed () const
{
    return m_treeSelProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasTreeItemRightClickProcessed () const
{
    return m_treeItemRightClickProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasNotebookPageSelProcessed () const
{
    return m_notebookPageSelProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasChoiceSelProcessed () const
{
    return m_choiceSelProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasCheckboxCheckingProcessed () const
{
    return m_checkboxChkgProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasRadioBoxSelProcessed () const
{
    return m_radioBoxSelProcessed;
}

bool WxGuiTestEvtSimHlpEvtHandler::HasSliderValChgProcessed () const
{
    return m_sliderChangeProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasSpinCtrlValChgProcessed () const
{
    return m_spinCtrlChangeProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasToolTogglingProcessed () const
{
    return m_toolTogglingProcessed;
}


bool WxGuiTestEvtSimHlpEvtHandler::HasCheckableToolTogglingProcessed () const
{
    return m_checkableToolTogglingProcessed;
}


void WxGuiTestEvtSimHlpEvtHandler::OnMenuItem (wxCommandEvent& event)
{
    m_menuItemSelProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnCheckableMenuItem (wxCommandEvent& event)
{
    m_chkMenuItemSelProcessed = !m_chkMenuItemSelProcessed;
}


void WxGuiTestEvtSimHlpEvtHandler::OnButtonClick (wxCommandEvent& event)
{
    m_buttonClickProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnTextCtrlChange (wxCommandEvent& event)
{
    m_textCtrlValChangeEvent = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnSpinCtrlDblChange (wxCommandEvent& event)
{
    m_spinCtrlDblValChangeEvent = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnTreeSelChange (wxTreeEvent &event)
{
    m_treeSelProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnTreeItemRightClick (wxTreeEvent &event)
{
    m_treeItemRightClickProcessed = true;

    wxMenu *m = new wxMenu ("PopupMenu");
    m->Append (1234, "PopupMenuItem");
    wxPoint p (0, 0);
    WxGuiTestHelper::PopupMenu ((wxTreeCtrl *) event.GetEventObject (), m,
            event.GetPoint (), "sccDatasetBrowserPopupMenu");
}


void WxGuiTestEvtSimHlpEvtHandler::OnNotebookPageChange (wxCommandEvent &event)
{
    m_notebookPageSelProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnChoiceSelChange (wxCommandEvent &event)
{
    m_choiceSelProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnCheckboxChecking (wxCommandEvent &event)
{
    m_checkboxChkgProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnRadioBoxSelChange (wxCommandEvent &event)
{
    m_radioBoxSelProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnSliderChange (wxCommandEvent &event)
{
    m_sliderChangeProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnSpinCtrlChange (wxCommandEvent &event)
{
    m_spinCtrlChangeProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnToggleTool (wxCommandEvent& event)
{
    m_toolTogglingProcessed = true;
}


void WxGuiTestEvtSimHlpEvtHandler::OnToggleCheckableTool (wxCommandEvent& event)
{
    m_checkableToolTogglingProcessed = true;
}

} // End namespace swTst

