///////////////////////////////////////////////////////////////////////////////
// Name:        swWxGuiTesting/swCRXRCReader.h
// Author:      Reinhold Freder
// Created:     2004
// Copyright:   (c) 2005 Reinhold Freder
// Licence:     wxWindows licence
///////////////////////////////////////////////////////////////////////////////

#ifndef SWCRXRCREADER_H
#define SWCRXRCREADER_H

#ifdef __GNUG__
    #pragma interface "swCRXRCReader.h"
#endif

#include "Common.h"

#include "swCRXRCResource.h"

namespace swTst {


/*! \class CRXRCReader
    \brief Read in and parse XRC files; returns XRC node hierarchy.

    If given filename is pointing to a valid directory instead of a single file
    the returned node hierarchy contains the nodes from all files (regular
    expression: "filename/*.xrc").
*/
class CRXRCReader
{
public:
    /*! \fn CRXRCReader (const wxString &filename)
        \brief Constructor

        \param filename name of XRC file to read, or path holding several XRC
            files of interest
    */
    CRXRCReader (const wxString &filename);


    /*! \fn virtual ~CRXRCReader ()
        \brief Destructor
    */
    virtual ~CRXRCReader ();


    /*! \fn virtual CRXRCResource * GetResource ()
        \brief Does the XRC file parsing and returns XRC node hierarchy.

        \return pointer to XRC node hierarchy (= XRC resource node)
    */
    virtual CRXRCResource * GetResource ();

protected:

private:
    wxString m_filename;

private:
    // No copy and assignment constructor:
    CRXRCReader (const CRXRCReader &rhs);
    CRXRCReader & operator= (const CRXRCReader &rhs);
};

} // End namespace swTst

#endif // SWCRXRCREADER_H

