///////////////////////////////////////////////////////////////////////////////
// Name:        swWxGuiTesting/swCRXRCSAXHandler.cpp
// Author:      Reinhold Freder
// Created:     2004
// Copyright:   (c) 2005 Reinhold Freder
// Licence:     wxWindows licence
///////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "swCRXRCSAXHandler.h"
#endif

#include "swCRXRCSAXHandler.h"

#include "xercesc/sax/AttributeList.hpp"

#include "swWxLogicErrorException.h"
#include "swXMLChStr.h"
#include "swCRXRCObject.h"

#include <iostream>

namespace swTst {


CRXRCSAXHandler::CRXRCSAXHandler ()
{
    m_expResource = true;

    m_contextNode = NULL;
}


CRXRCSAXHandler::~CRXRCSAXHandler ()
{
    // Nothing to do
}


void CRXRCSAXHandler::startElement (const XMLCh * const name,
        xercesc::AttributeList &attributes)
{
    wxString elemName = sw::XMLChStr (name).CharForm ();
    //std::cout << "I saw element: "<< elemName.c_str () << std::endl;

    if (m_expResource) {

        if (elemName.MakeLower () != "resource") {

            throw sw::WxLogicErrorException (_("First element must be <resource>"));
        }
        m_expResource = false;

    } else if (elemName == "object") {

        wxString objName = sw::XMLChStr (attributes.getValue ("name")).CharForm ();
        wxString objClass = sw::XMLChStr (attributes.getValue ("class")).CharForm ();

        CRXRCObject *object = new CRXRCObject (objName, objClass,
                m_contextNode);
        m_contextNode->AddChild (object);

        m_contextNode = object;
    }
}


void CRXRCSAXHandler::endElement (const XMLCh * const name)
{
    wxString elemName = sw::XMLChStr (name).CharForm ();

    if (elemName == "object") {

        if (!m_contextNode->GetParent ()) {

            throw sw::WxLogicErrorException (
                    _("Fatal parsing error: closing <object> tag without a beginning tag"));
        }
        m_contextNode = m_contextNode->GetParent ();
    }
}


void CRXRCSAXHandler::fatalError (const xercesc::SAXParseException &exception)
{
    wxString message = sw::XMLChStr (exception.getMessage ()).CharForm ();
    
    std::cout << "Fatal Error: " << message.c_str ()
         << " at line: " << exception.getLineNumber ()
         << std::endl;

    throw sw::WxLogicErrorException (wxString::Format (
            _("Fatal parsing error: %s (line: %d)"), message.c_str (),
            exception.getLineNumber ()));
}


void CRXRCSAXHandler::SetContextNode (CRXRCNode *contextNode)
{
    m_contextNode = contextNode;
}

} // End namespace swTst

