///////////////////////////////////////////////////////////////////////////////
// Name:        swWxGuiTesting/swCRXRCSAXHandler.h
// Author:      Reinhold Freder
// Created:     2004
// Copyright:   (c) 2005 Reinhold Freder
// Licence:     wxWindows licence
///////////////////////////////////////////////////////////////////////////////

#ifndef SWCRXRCSAXHANDLER_H
#define SWCRXRCSAXHANDLER_H

#ifdef __GNUG__
    #pragma interface "swCRXRCSAXHandler.h"
#endif

#include "Common.h"

#include <xercesc/sax/HandlerBase.hpp>

namespace swTst {

class CRXRCNode;


/*! \class CRXRCSAXHandler
    \brief SAX handler for reading XRC files (object node elements).
*/
class CRXRCSAXHandler : public xercesc::HandlerBase
{
public:
    /*! \fn CRXRCSAXHandler ()
        \brief Constructor
    */
    CRXRCSAXHandler ();


    /*! \fn virtual ~CRXRCSAXHandler ()
        \brief Destructor
    */
    virtual ~CRXRCSAXHandler ();


    /*! \fn virtual void startElement (const XMLCh * const name, xercesc::AttributeList &attributes)
        \brief Receive notification of the start of an element.

        \param name element type name
        \param attributes specified or defaulted attributes
    */
    virtual void startElement (const XMLCh * const name,
            xercesc::AttributeList &attributes);


    /*! \fn virtual void startElement (const XMLCh * const name, xercesc::AttributeList &attributes)
        \brief Receive notification of the end of an element.

        \param name element type name
    */
    virtual void endElement (const XMLCh * const name);


    /*! \fn virtual void fatalError (const xercesc::SAXParseException &exception)
        \brief Report fatal error message.

        \param exception detected fatal error description
    */
    virtual void fatalError (const xercesc::SAXParseException &exception);


    /*! \fn void SetContextNode (CRXRCNode *contextNode)
        \brief Set context node for building the XRC element hierarchy.

        \param contextNode current context node making up the parent node for newly found elements
    */
    void SetContextNode (CRXRCNode *contextNode);

protected:

private:
    bool m_expResource;

    CRXRCNode *m_contextNode;

private:
    // No copy and assignment constructor:
    CRXRCSAXHandler (const CRXRCSAXHandler &rhs);
    CRXRCSAXHandler & operator= (const CRXRCSAXHandler &rhs);
};

} // End namespace swTst

#endif // SWCRXRCSAXHANDLER_H

